/*
The search algorithm can be coded very concisely in C, as a function
defined in a single "statement", as follows:

unsigned char romsearch()
{
   unsigned char xi = 0, x = 0, ld = l0, tr;

   if (tr = touchreset(pn) && touchbyte(0xf0))
      while (xi++ < 64 && (x = ((touchbit(1) << 1) + touchbit(1))) < 3)
         l0 = !touchbit((bits[xi] = !x ? xi == ld : ~x & 1)) && !x ? xi : l0;
   return (x == 3) || !tr ? 0 : 1 + (ld == l0);
}

In order to use the above search algorithm, the bits of the ROM serial number
must be stored one bit per array element in the array named "bits". The only
other global variable used in this function is pn which represents the com
port number. Because the code is very concise, it may be difficult to follow
the logic of its operation. A second example, shown below, is more readily
understood.

The following C code implements the First and Next functions as described in
the flowchart of figure 5-3. In addition, the code shown below provides two
additional features by checking the Dallas 1-Wire CRC of the ROM data (i.e.,
it will only find parts with a good CRC) and by including a search for a
DS1990 after all multidrop 1-wire parts have been found. This allows the
DS1990 to appear compatible with a multidrop 1-wire bus, provided that only
one DS1990 is connected to the bus.

The correspondence between variable names used in the Next procedure and the
identifiers used in the flowchart of figure 5-3 are as follows:

Last Discrepancy   : LD
Return Value       : NXT
Done Flag          : LastOne
ROM Bit Index      : I
Discrepancy Marker : L0
Output Bit         : G
ROM Bit            : ROM

Note that ROM is a byte array with starting index of zero, whereas ROM Bit
(as used in the flowchart) is a bit array with starting index of one. The
byte index J and the bit mask K are used in the program code to address the
individual bits of the byte array as if it were an array of bits.

*/

/* Type definition */
typedef unsigned char uchar;

/* External Variable declared in calling program */
extern uchar DOWCRC;

/* Externally defined functions */
extern uchar TouchByte( uchar );
extern uchar TouchReset( void );
extern uchar TouchBit(uchar);

/* Function Prototypes */
uchar Next( void );
uchar First( void );
uchar Access90( void );
uchar dowcrc(uchar);

/* Global Variables */
const char TR = 1;  /* TRUE  */
const char FL = 0;  /* FALSE */
uchar ROM[8];
uchar LD = 0;  /* Last bit discrepancy in previous Next */
uchar LastOne=0;
uchar Got90=0;


//////////////////////////////for test
#include <stdio.h>

extern uchar Setup(uchar);
extern uchar com_port;

void main(void)
{
    int i;

    printf("setup %d\n",Setup(com_port));

    printf("First %d\n",First());

    for (i = 0; i < 8; i++)
        printf("%02X ",ROM[i]);

    printf("\n");
    printf("Next %d\n",Next());

    for (i = 0; i < 8; i++)
        printf("%02X ",ROM[i]);
    printf("\n");
}
//////////////////////////////for test




/*------------------------------------------------------------------------
 * The 'First' function simply resets the current state of a ROM search
 * and calls next to find the first device on the 1-Wire.
 */
uchar First( void )
{
     LD = 0;            /* reset the rom search Last Discrepancy Global */
     Got90   = FL;
     LastOne = FL;
     return Next();            /* call next and return its return value */
}

/*------------------------------------------------------------------------
 * The 'Next' function searches for the next device on the 1-wire. If
 * a multi-drop (Ds1991,2,3 or 4) is not found then a single DS1990 is
 * then searched for. If the previous 'Next' was a DS1990 or there
 * are no more devices on the 1-wire then FL is returned.
 */
uchar Next( void )
{
     uchar I=1,J=0,K=1,X,LO=0,G,NXT;
     int flag;

     NXT = FL;                            /* set the next flag to false */
     DOWCRC = 0;                                    /* reset the DOWCRC */
     if (Got90)         /* check to see if previous Next found a DS1990 */
     {
            /* since DS1990 is done last then last Next was last device */
          Got90 = FL;
          return FL;
     }
     /* if previous Next was the last multi-drop then look for a DS1990 */
     if (LastOne)   /* if last Next was last multidrop, look for DS1990 */
          LastOne = FL;
     else
     {
          flag = TouchReset();                      /* reset the 1-wire */
          if (!(flag == 1 || flag == 2))       /* no parts => return FL */
          {
               LD = 0;        /* reset the search */
               return FL;
          }
          X = TouchByte(0xF0);          /* issue the search rom command */
          do            /* loop to do the search rom for DS1991,2,3 or 4*/
          {
               X = TouchBit(1) << 1;                      /* read a bit */
               X |= TouchBit(1);                  /* and its complement */
               if (X == 3)            /* there are no devices on 1-wire */
                  break;
               else
               {
                    if (X > 0)       /* all devices coupled have 0 or 1 */
                       G = X >> 1;        /* bit write value for search */
                    else
                    {
                            /* If this discrepancy is before the Last
                            Discrepancy on a previous next then pick
                            the same as last time */
                         if (I < LD)
                            G = ((ROM[J] & K) > 0);
                         else                /* if equal to last pick 1 */
                            G = (I == LD);   /* if not then pick 0      */

                          /* if 0 was picked then record position in LO */
                         if (G == 0) LO = I;
                    }
                    if (G == 1)    /* isolate bit in ROM[J] with mask K */
                       ROM[J] |= K;
                    else
                       ROM[J] &= ~K;
                    G = TouchBit(G);                /* rom search write */

                    I++;                    /* increment byte counter I */
                    K = K << 1;             /* and shift the bit mask K */
                    /* if the mask is 0 then go to
                    new ROM byte J and reset mask */
                    if (K == 0)
                    {
                         dowcrc(ROM[J]);          /* accumulate the CRC */
                         J++; K++;
                    }
               }
          } while(J < 8);       /* loop until through all ROM bytes 0-7 */
          if (I < 65 || DOWCRC)      /* if search was unsuccessful then */
             LD = 0;                 /* reset the Last Discrepancy to 0 */
          else
          {
               /* search successful so set LD,LastOne,NXT */
               LD = LO;
               LastOne = (LD == 0);
               NXT = TR;
          }
     }
     if (!NXT)  /* a DS1991,2,3, or 4 was not found so try for a DS1990 */
     {
          if (Access90())
          {
             Got90 = TR;                 /* found a DS1990 so set Got90 */
             NXT = TR;
          }
     }
     return NXT;
}

/*------------------------------------------------------------------------
 * The 'Access90' function resets the 1-Wire and sends a DS1990 READ ROM
 * command to a single DS1990 on the 1-Wire. If this fails at any time
 * in the process then a FL is returned, otherwise TR is returned.
 */
uchar Access90( void )
{
     int i,flag;

     flag = TouchReset();
     if (flag == 1 || flag == 2)               /* if presence on 1-wire */
     {
          if (TouchByte(0x0F) != 0x0F)  /* send DS1990 READ ROM command */
             return FL;
          DOWCRC = 0;                               /* reset the DOWCRC */
          for (i = 0; i <= 7; i++)                      /* Read the ROM */
          {
              ROM[i] = TouchByte(0xFF);
              dowcrc(ROM[i]);
          }
          if (DOWCRC == 0)
             return TR;
          else
             return FL;
     }
     else
          return FL;
}

/*------------------------------------------------------------------------
 * Update the Dallas Semiconductor One Wire CRC (DOWCRC) from the global
 * variable DOWCRC and the argument. Return the updated DOWCRC.
 */
uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

uchar dowcrc(uchar x)
{
   DOWCRC = dscrc_table[DOWCRC ^ x];
   return DOWCRC;
}


