/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * TOUCHIO.C - C program module containing Low level I/O 1-Wire
 *             functions for DS199X Touch Memory utilities.
 *             Version 2.2.
 *
 *     'setup' initializes the com port for use.
 *
 *     'TouchReset','TouchByte' and 'TouchBit' are Link-Level 1-Wire
 *           routines.
 *
 *     'First','Next','Access','Access90','RomData','FirstAlarm','NextAlarm'
 *           are Network-Level 1-Wire routines.
 *
 *     'crc16' and 'dowcrc' are cyclic redundancy routines.
 *
 * Compiler: Borland C++ 3.1
 * Version: 2.2
 *
 *
 * NOTE: To use Microsoft C compilers, include the following lines at the
 *       top of this source file.
 *          #define inportb   inp
 *          #define outportb  outp
 *
 */

#include <dos.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned long ulong;

/* External Variables declared in calling program */
extern int   com_port;
extern uint  CRC16;
extern uchar DOWCRC;

/* Function Prototypes */
uchar Setup( uchar );
uchar TouchByte( uchar );
uchar TouchReset( void );
uchar TouchBit(uchar);
uchar First( void );
uchar Next( void );
uchar FirstAlarm( void );
uchar NextAlarm( void );
uchar GenFirst(uchar);
uchar GenNext(uchar);
uchar Access( void );
uchar Access90( void );
uchar * RomData( void );
uint  crc16(int);
uchar dowcrc(uchar);

/* Global Variables for TOUCHIO.C */
const char TR = 1;  /* TRUE  */
const char FL = 0;  /* FALSE */
uchar ROM[8];
uchar LD = 0;  /* Last bit discrepancy in previous Next */
static uchar LastOne=0;
static uchar Tried90=0;

/*--------------------------------------------------------------------------
 * The setup functions makes sure that the com port number passed to it
 * is from 1 to 4 and has a valid address associated with it.
 */
uchar Setup(uchar CmPt)
{
     uint far *ptr = (uint far *) 0x00400000;
     uint SPA;

     /* check to see if it is a valid com port number and address */
     SPA = *(ptr+CmPt-1); /* get the address */
     if (CmPt < 1 || CmPt > 4 || !SPA )
        return FL;

     /* serial port initialization */
     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+1,0x00);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA+1,0x00);  /* no interrupts */
     outportb(SPA+4,0x03);  /* RTS and DTR on */

     return TR;
}


/*--------------------------------------------------------------------------
 * Do a reset on the 1 wire port and return  0 no presence detect
 *                                           1 presence pulse no alarm
 *                                           2 alarm followed by a presence
 *                                           3 short circuit to ground
 *                                           4 no com port found
 *
 * The global variable 'com_port' must be set to the com port that
 * the serial brick is attached to before calling this routine.
 *
 */
uchar TouchReset( void )
{
     uint SPA,F,X,Y,tmp,trst=0;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* return if there is no address */
     if (!SPA) return 4;

     /* serial port initialization */
     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+1,0x00);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA+1,0x00);  /* no interrupts */
     outportb(SPA+4,0x03);  /* RTS and DTR on */

     /* Initialize the time limit */
     M = *sysclk + 1;

     /* loop to clear the buffers */
     do { tmp = inportb(SPA+5) & 0x60;  } while (tmp != 0x60);

     /* flush input */
     while (inportb(SPA+5) & 0x1) X = inportb(SPA);

     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA+1,0x00);  /* baud rate is 10473 */
     outportb(SPA  ,0x0B);
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */
     outportb(SPA  ,0xF0);  /* send the reset pulse */

     /* wait until character back or timeout */
     do
     {
          Y = inportb(SPA+5);
          F = Y & 0x1;
     } while ( !F && (*sysclk <= M) );


     if (F) X = inportb(SPA);
     else   return 3;

     if (X != 0xF0)       /* if more bits back than sent then there is a */
     {                    /* device  if framing error or break */
          trst = TR;
          if ( (Y & 0x18) != 0 )
          {
               trst = 2;

               /* loop to clear the buffers */
               do { tmp = inportb(SPA+5) & 0x60;  } while (tmp != 0x60);

               /* wait until character back or timeout */
               do
               {
                    Y = inportb(SPA+5);
                    F = Y & 0x1;
               } while ( !F && (*sysclk <= M) );

               if (F) X = inportb(SPA);
               else return 3;
          }
     }

     outportb(SPA+3,0x83);  /* set DLAB */
     outportb(SPA  ,0x01);  /* bit rate is 115200 */
     outportb(SPA+3,0x03);  /* 8 dta, 1 stp, no par */

     return trst;
}


/*--------------------------------------------------------------------------
 * This is the 1-Wire routine 'TouchByte' or sometimes called 'databyte'.
 * It transmits 8 bits on to the 1-Wire data line and receives 8 bits
 * concurrently. The global variable 'com_port' must be set to the
 * com port that the serial brick is attached to before calling this
 * routine.  This com port must also be set to 115200 baud, 8 dta, 1 stp,
 * and no parity.  This routine returns the uchar 8 bit value received.
 * If it times-out waiting for a character then 0xFF is returned.
 */
uchar TouchByte(uchar outch)
{
     uchar inch=0,sendbit,Mask=1;
     uint SPA;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* Initialize the time limit */
     M = *sysclk + 2;

     /* wait to TBE and TSRE */
     do {} while ( (inportb(SPA+5) & 0x60) != 0x60 );

     /* flush input */
     while ( (inportb(SPA+5) & 0x1) )
          inportb(SPA);

     /* get first bit ready to go out */
     sendbit = (outch & 0x1) ? 0xFF : 0x00;

     /* loop to send and receive 8 bits */
     do
     {
          outportb(SPA,sendbit);  /* send out the bit */

          /* get next bit ready to go out */
          Mask <<= 1;
          sendbit = (outch & Mask) ? 0xFF : 0x00;

          /* shift input char over ready for next bit */
          inch >>= 1;

          /* loop to look for the incoming bit */
          for (;;)
          {
               /* return if out of time */
               if ( *sysclk > M )
                  return 0xFF;

               if ( inportb(SPA+5) & 0x01 )
               {
                  inch |= ((inportb(SPA) & 0x01) ? 0x80 : 0x00);
                  break;
               }
          }

     } while (Mask);

     return inch;   /* return the input char */
}


/*--------------------------------------------------------------------------
 * This is the 1-wire routine 'TouchBit'.  It is used in the search rom
 * and match rom operations.  It sends out one bit on the 1-wire and
 * returns one bit.  The global variable 'com_port' must be set to the
 * com port that the serial brick is attached to before calling this
 * routine.  This com port must also be set to 115200 baud, 8 dta, 1 stp,
 * and no parity.  If this routine times-out waiting for the bit character
 * then 0x01 is returned.
 */
uchar TouchBit(uchar outbit)
{
     uchar sendbit;
     uint SPA;
     uint far *ptr = (uint far *) 0x00400000;
     ulong far *sysclk = (ulong far *) 0x0040006c;
     ulong M;

     /* get the serial port address */
     SPA = *(ptr+com_port-1);

     /* Initialize the time limit */
     M = *sysclk + 2;

     /* wait to TBE and TSRE */
     do {} while ( (inportb(SPA+5) & 0x60) != 0x60 );

     /* flush input */
     while ( (inportb(SPA+5) & 0x1) )
          inportb(SPA);

     /* get bit ready to go out */
     sendbit = (outbit & 0x1) ? 0xFF : 0x00;

     outportb(SPA,sendbit);  /* send out the bit */

     /* loop to look for the incoming bit */
     for (;;)
     {
          /* return if out of time */
          if ( *sysclk > M )
             return 1;

          if ( inportb(SPA+5) & 0x01 )
               return (inportb(SPA) & 0x01);  /* return the input bit */
     }
}


/*--------------------------------------------------------------------------
 * The 'First' function calls the general first function 'GenFirst' with
 * alarm search flag set to false so that it finds the first device on
 * the 1-Wire without regard to alarming conditions.
 */
uchar First( void )
{
     /* call general first without regard to alarming conditions */
     return GenFirst(FL);
}


/*--------------------------------------------------------------------------
 * The 'Next' function calls the general next function 'GenNext' with
 * alarm search flag set to false so that it finds the next device on
 * the 1-Wire without regard to alarming conditions.
 */
uchar Next( void )
{
     /* call general next without regard to alarming conditions */
     return GenNext(FL);
}


/*--------------------------------------------------------------------------
 * The 'FirstAlarm' function calls the general first function 'GenFirst'
 * with alarm search flag set to true so that it finds the first alarming
 * device on the 1-Wire.
 */
uchar FirstAlarm( void )
{
     /* call general first with alarming search flag on */
     return GenFirst(TR);
}


/*--------------------------------------------------------------------------
 * The 'NextAlarm' function calls the general next function 'GenNext'
 * with alarm search flag set to true so that it finds the next alarming
 * device on 1-Wire.
 */
uchar NextAlarm( void )
{
     /* call general next with alarming search flag on */
     return GenNext(TR);
}


/*--------------------------------------------------------------------------
 * The 'GenFirst' function simply resets the current state of a ROM search
 * and calls GenNext to find the first device on the 1-Wire. 'AlSrch'
 * specifies if it is an alarm search only or a regular search.
 */
uchar GenFirst(uchar AlSrch)
{
     /* reset the rom search Last Discrepancy Global */
     LD = 0;
     Tried90 = FL;
     LastOne = FL;

     /* call next and return it's return value */
     return GenNext(AlSrch);
}


/*--------------------------------------------------------------------------
 * The 'GenNext' function searchs for the next device on the 1-wire.  If
 * a multi-drop (Ds1991,2,3 or 4) is not found and there has not been a
 * multi-drom DS1990 found then a single DS1990 is then searched for.
 */
uchar GenNext(uchar AlSrch)
{
     uchar I=1,J=0,K=1,X,LO=0,G,NXT;
     int flag;

     NXT = FL;   /* set the next flag to false */
     DOWCRC = 0;     /* reset the DOWCRC */


     /* if the last call was not the last one */
     if (!LastOne)
     {
          /* reset the 1-wire */
          flag = TouchReset();

          /* if there are no parts or alarming parts on 1-wire, return FL*/
          if (!(flag == 1 || flag == 2))
          {
               LD = 0;        /* reset the search */
               return FL;
          }

          if (AlSrch)
             X = TouchByte(0xEC); /* issue the alarming search rom command */
          else
             X = TouchByte(0xF0); /* issue the search rom command */

          /* loop to do the search rom for DS1991,2,3 or 4*/
          do
          {
               /* read a bit and its compliment */
               X = TouchBit(1) << 1;
               X |= TouchBit(1);

               if (X == 3) /* there are no devices on 1-wire */
                  break;
               else
               {
                    if (X > 0)           /* all devices coupled have 0 or 1 */
                       G = !(X & 0x01);  /* bit write value for search */
                    else
                    {
                         /* if this discrepancy if before the Last Discrepancy
                            on a previous next then pick the same as last time */
                         if (I < LD)
                            G = ((ROM[J] & K) > 0);
                         else              /* if equal to last pick 1 */
                            G = (I == LD); /* if not then pick 0      */

                         /* if 0 was picked then record its position in LO */
                         if (G == 0) LO = I;
                    }

                    /* set or clear the bit in the ROM byte J with mask K */
                    if (G == 1)
                       ROM[J] |= K;
                    else
                       ROM[J] &= ~K;

                    /* rom search write */
                    G = TouchBit(G);

                    /* increment the byte counter I and shift the mask K */
                    I++;
                    K = K << 1;

                    /* if the mask is 0 then go to new ROM byte J and reset mask */
                    if (K == 0)
                    {
                         dowcrc(ROM[J]);  /* accumulate the CRC */
                         J++; K++;
                    }
               }

          } while(J < 8);  /* loop until through all ROM bytes 0-7 */

          if (I < 65 || DOWCRC)    /* if the search was unsuccessful then */
             LD = 0;              /* reset the Last Discrepancy to 0      */
          else
          {
               /* search successful to set LD,LastOne,NXT */
               LD = LO;
               LastOne = (LD == 0);
               NXT = TR;
               /* if is a DS1990 then set Tried90 */
               if (ROM[0] == 0x01)
                  Tried90 = TR;
          }
     }

     /* a DS1991,2,3, or 4 was not found so try for a DS1990 */
     if (!NXT && !Tried90 && !AlSrch)
     {
          Tried90 = TR;
          /* attempt to access a DS1990 */
          if (Access90())
             NXT = TR;
     }

     /* if no device found then reset counters so next 'next' will be
        like a first */
     if (!NXT)
     {
          LD = 0;
          Tried90 = FL;
          LastOne = FL;
     }

     return NXT;
}


/*--------------------------------------------------------------------------
 * The 'Access' function resets the 1-Wire and sends a MATCH ROM command
 * followed by the ROM code.  If this fails at any time in the process
 * then a FL is returned, otherwise TR is returned.
 */
uchar Access( void )
{
     int flag,i;

     flag = TouchReset();          /* reset the 1-wire */

     if (flag == 1 || flag == 2)   /* if presence or alarm presence */
     {
          /* send the MATCH ROM command */
          if (TouchByte(0x55) != 0x55)
             return FL;

          /* send the ROM to match */
          for (i = 0; i <= 7; i++)
          {
               if (TouchByte(ROM[i]) != ROM[i])
                  return FL;
          }

          return TR;
     }
     else
          return FL;
}


/*--------------------------------------------------------------------------
 * The 'Access90' function resets the 1-Wire and sends a DS1990 READ ROM
 * command to a single DS1990 on the 1-Wire.  If this fails at any time
 * in the process then a FL is returned, otherwise TR is returned.
 */
uchar Access90( void )
{
     int i,flag;

     flag = TouchReset();
     if (flag == 1 || flag == 2)   /* if presence on 1-wire */
     {
          /* send the DS1990 READ ROM command */
          if (TouchByte(0x0F) != 0x0F)
             return FL;

          DOWCRC = 0;  /* reset the DOWCRC */

          /* Read the ROM */
          for (i = 0; i <= 7; i++)
          {
              ROM[i] = TouchByte(0xFF);
              dowcrc(ROM[i]);
          }

          if (DOWCRC == 0)
             return TR;
          else
             return FL;
     }
     else
          return FL;
}



/*--------------------------------------------------------------------------
 * The 'RomData' function returns the pointer to the ROM DATA.
 */
uchar * RomData( void )
{
     return &ROM[0];
}


/*--------------------------------------------------------------------------
 * Calculate a new CRC16 from the input data integer.  Return the current
 * CRC16 and also update the global variable CRC16.
 */
static int oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

uint crc16(int data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
      CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}


uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};


/*--------------------------------------------------------------------------
 * Update the Dallas Semiconductor One Wire CRC (DOWCRC) from the global
 * variable DOWCRC and the argument.  Return the updated DOWCRC.
 */
uchar dowcrc(uchar x)
{
   DOWCRC = dscrc_table[DOWCRC ^ x];
   return DOWCRC;
}

