/*
   The following C code demonstrates reading and writing touch memory
   (DS1992, DS1993, DS1994) using a universal data packet(length, data,
   crc16 ones complemented).

      Copyright (C) 1992 Dallas Semiconductor Corporation

   Syntax:  WR_RAM (com port #) (starting page #) ("write string")
*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;

/* Global Variables */
const char TRUE   = 1;
const char FALSE  = 0;
/* Global Variables needed by TOUCHIO.OBJ */
int   com_port;      /* serial com port for DOW */
uint  CRC16;         /* 16-bit cyclic redundancy check */
uchar DOWCRC;        /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* External Function Prototypes from 'TOUCHIO.OBJ' */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar TouchByte( uchar );
extern uchar Access( void );
extern uchar First( void );
extern uchar * RomData( void );
extern uint  crc16(int);

/* Function Prototypes for WR_RAM.C */
void main(int, char **);
int  Open_Port( int );
int  Read_Ram_String(uchar *, uint);
int  Write_Ram_String(uchar *, uint);
int  Format_String(uchar *, uchar *, uint);
int  Read_Scratchpad(uchar *, uint *, uint *);
int  Write_Scratchpad(uchar *, uint, uint);
int  Copy_Scratchpad(uint addr, uint offset);
int  Array_Comp(uchar *, uchar *, int);


/*--------------------------------------------------------------------------
 * This is the main routine in WR_RAM.C.
 */
void main(int argc, char **argv)
{
     int page,flag;
     uchar *wr_str;

     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                     WR_RAM - Version 1.0\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port and page number from the command line parameters*/
     if (argc != 4)
     {
           printf("\nSYNTAX: WR_RAM (com port #) (starting page #)"
                  "(\"write string\")\n");
           exit(1);
     }
     else
     {
          com_port = atoi(argv[1]);
          page = atoi(argv[2]);
          wr_str = (uchar*)(argv[3]);
     }

     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     if (!First())
        printf("                   No Touch Memory on COM%d!\n"
               "    Touch Memory contact required during program"
               "execution.\n\n",com_port);
     else
     {
          flag = Write_Ram_String(wr_str, page << 5);

          if (flag > FALSE)
          {
               printf("A string of length %d was written to the device\n"
                      "starting on page %d. It is enclosed in {}.\n\n"
                      "{%s}\n\n",strlen((char*)(wr_str)),page,wr_str);
          }
          else if (flag == FALSE)
               printf("There was an error in writing to the device.\n\n");
          else
               printf("Contact was lost before write and verify was "
                      "complete.\n\n");
     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on "
             "this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the "
             "chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 * Return up to 508 bytes of the non-volatile ram of a selected 92,93,94
 * starting at a provided address. Returns the number of the bytes
 * successfully received.
 */
int Read_Ram_String(uchar *buf, uint addr)
{
     uint i,len,cnt=0;

     buf[0] = NULL;                 /* zero the output buffer */

     if (!Access())                 /* select the device that was found */
        return FALSE-1;

     TouchByte(0xF0);                /* read memory command */

     TouchByte(addr & 0xFF);         /* write the target address byte 1 */

     TouchByte((addr & 0xFF00) >> 8);/* write the target address byte 2 */

     CRC16 = addr >> 5;              /* seed the CRC16 with the page # */

     len = TouchByte(0xFF);          /* read the first length byte */

     crc16(len);                     /* compute CRC16 on first byte */

     if (len == 255)
     {
          len = len + TouchByte(0xFF);
                             /* if = 255 then read second length byte */
          crc16(len-255);             /* compute crc16 on second byte */
     }

     if (len > (510-addr))          /* abort read if length to large */
     {
        if (!Access())              /* verify that length was not  */
           return FALSE-1;          /* because connection was lost */
        else
           return FALSE;
     }

     for (i = 0; i < len+2; ++i)    /* loop to read record */
     {
          buf[cnt] = TouchByte(0xFF);
          crc16(buf[cnt++]);
     }

     buf[cnt-2] = NULL;             /* zero end the string over the CRC */

     if (CRC16 == 0xB001)           /* check to see if CRC is correct */
        return cnt-2;
     else
     {
        if (!Access())               /* verify that bad string was not */
           return FALSE-1;           /* because connection was lost    */
        else
           return FALSE;
     }
}


/*--------------------------------------------------------------------------
 * Writes up to 508 characters to NV-sram. If there is an error during
 * write or the string can not fit in the space then FALSE is returned.
 * If the write is done and verified then TRUE is returned.
 */
int Write_Ram_String(uchar *buf, uint addr)
{
     uint  i,j,Num_Pages,cnt=0,end_page,flag;
     uchar wrbuf[513],tmp[513],*ROM;
     int   len;

     len = Format_String(buf,wrbuf,addr >> 5); /* format buf into wrbuf */

     Num_Pages = ((len-1) >> 5)+1;   /* calculate number of pages needed */

     ROM = RomData();  /* get pointer to ROM CODE */

     end_page = Num_Pages + (addr >> 5); /* calculate end page */

     /* check to make sure length and position of write is valid */
     if (ROM[0] == 0x08 && end_page > 4) /* DS1992 has 4 pages */
     {
          printf("\nERROR: Write String length goes over 4 page "
                 "limit of DS1992!\n");
          exit(1);
     }
     else if (ROM[0] == 0x01 || ROM[0] == 0x02)
     {
          printf("\nERROR: This demo program is written for DS1992,3 and "
                 "4's.  The part\n"
                 "       detected is not one of these.\n");
          exit(1);
     }
     else if (end_page > 16)
     {
          printf("\nERROR: Write String length goes over 16 page limit!\n\n");
          exit(1);
     }

     for (i = 0; i < Num_Pages; i++) /* loop for each page to write */
     {
          for(j = 0; j < 32; j++)    /* copy part of write buf to temp*/
             tmp[j] = wrbuf[cnt++];

          tmp[j] = 0;
                                     /* write and copy the scratchpad */
          flag = Write_Scratchpad(tmp,addr+i*0x20,31);
          if ( flag == TRUE)
             Copy_Scratchpad(addr+i*0x20,31);
          else
             return flag;
     }

     len = Read_Ram_String(tmp,addr);
     if (len > 0)                    /* check the ram to verify write */
     {
         if (strcmp((char*)(buf),(char*)(tmp)))
            return FALSE;
     }
     else
         return len;

     return TRUE;
}


/*--------------------------------------------------------------------------
 * Format a string with the a single or double length byte in the front
 * and a CRC16 double byte appended to the end. Returns the total length
 * of the formatted string.
 */
int Format_String(uchar *buf, uchar *fbuf, uint page)
{
     uint len,i,cnt=0;

     len = strlen((char*)(buf));          /* get length of the string */

     CRC16 = page;                        /* seed the CRC16 with page # */

     if (len < 255)                 /* set length in formatted buffer */
     {
        fbuf[cnt++] = len;
        crc16(len);
     }
     else
     {
        fbuf[cnt++] = 255;
        crc16(255);
        fbuf[cnt++] = len - 255;
        crc16(len - 255);
     }

     for (i = 0; i < len; i++)      /* copy the buffer over to the fbuf*/
     {
          fbuf[cnt++] = buf[i];
          crc16(buf[i]);
     }

     fbuf[cnt++] = ~(CRC16 & 0xFF);  /* append the crc to formatted buffer */
     fbuf[cnt++] = ~((CRC16 & 0xFF00) >> 8);

     fbuf[cnt] = 0;                /* end the string */

     return cnt;                   /* return the length of fbuf */
}


/*--------------------------------------------------------------------------
 * Provides up to 32 bytes of the scratch pad of a selected 92,93,94 and
 * also sets the target address 'addr' and the offset. The Number
 * of bytes can be calculated from the 'addr' and 'offset'.
 */
int Read_Scratchpad(uchar *buf, uint *addr, uint *offset )
{
     int i,lpcnt=0,num_bytes,flag;
     uchar ch;

     /* loop until get the SCRATCHPAD free of error or 4 times max */
     do
     {
          flag = TRUE;                /* set the flag to true */

          if (!Access())         /* make sure still have a device */
             return FALSE-1;

          TouchByte(0xAA);             /* read scratchpad command */

          *addr = TouchByte(0xFF);     /* read the target address byte 1 */

                                      /* read the target address byte 2 */
          *addr = *addr | (TouchByte(0xFF) << 8);

          *offset = TouchByte(0xFF);   /* read the target address offset */

          /* calculate the number of bytes in scratchpad 1-32 */
          num_bytes = 
             ((*addr & 0xE0) | (*offset & 0x1F)) - (*addr & 0xFF) + 1;

          for (i = 0; i < num_bytes; ++i)    /* read scratchpad contents */
          {
              ch = TouchByte(0xFF);   /* read byte */
              if (ch != buf[i]) flag = FALSE; /* compare with old set flag*/
              buf[i] = ch;
          }

     } while (lpcnt++ < 3 && flag == FALSE);

     if (flag == TRUE)
        return TRUE;
     else if (!Access())   /* verify that the fail was not because */
        return FALSE-1;    /* contact was lost */
     else
        return FALSE;
}


/*--------------------------------------------------------------------------
 * Write 32 bytes of the scratch pad of a selected 92,93,94 . Returns true
 * if Access() returns true.
 */
int Write_Scratchpad(uchar *buf, uint addr ,uint offset)
{
   int   i;
   uint  taddr,toffset;
   uchar tbuf[33];

   if (!Access())       /* select the device that was found */
      return FALSE-1;

   TouchByte(0x0F);                /* write scratchpad command */

   TouchByte(addr & 0xFF);         /* write the target address byte 1 */

   TouchByte((addr & 0xFF00) >> 8);/* write the target address byte 2 */

                                   /* write num bytes scratchpad contents */
   for (i = 0; i <= (offset-(addr & 0x1F)); i++)
      TouchByte(buf[i]);

   /* Check to make sure the data has been written correctly by reading */
   i = Read_Scratchpad(tbuf,&taddr,&toffset);
   if (i == TRUE)
      if (Array_Comp(buf,tbuf,i) &&       /* data same? */
          (addr == taddr) &&              /* target address same? */
          (offset == toffset) )           /* offset same? */
         return TRUE;
      else
         return FALSE;
   else
      return i;
}


/*--------------------------------------------------------------------------
 * Send the copy command to copy the scratchpad to NV ram.
 */
int Copy_Scratchpad(uint addr, uint offset)
{
     if (!Access())        /* select the device that was found */
        return FALSE-1;

     TouchByte(0x55);                 /* sent the copy command */

     TouchByte(addr & 0xFF);          /* write the target address byte 1 */

     TouchByte((addr & 0xFF00) >> 8); /* write the target address byte 2 */

     TouchByte(offset);               /* send the offset byte */

     return TRUE;
}


/*--------------------------------------------------------------------------
 * Compares two unsigned character arrays and returns TRUE
 * if they are the same
 */
int Array_Comp(uchar *ar1, uchar *ar2, int len)
{
     int i=0;

     if (len == -1)
        len = strlen((char*)(ar1));

     for (i = 0; i < len; i++)
     {
         if (ar1[i] != ar2[i])
            return FALSE;
     }

     return TRUE;
}

