Program TouchInt;                   { Ver 1.03  SMC  March 11, 1992 }
{
   This program demonstrates the use of the presence pulse from a
   Touch Memory to invoke a hardware interrupt procedure which
   reads and displays the ROM data.
}

{$R-}                                      { Inhibit range checking }
{$S-}                                      { Inhibit stack checking }
{$I-}                                        { Inhibit I/O checking }
{$V-}                                       { Relaxed type checking }
{$B-}                            { Short circuit boolean evaluation }
{$N-}                          { Software floating point evaluation }
{$D-}                                      { Omit debug information }
{$L-}                                          { Omit local symbols }
{$O-}                                       { No overlay capability }
{$A+}                                       { Select word alignment }
{$E-}                                      { Inhibit 8087 emulation }
{$M 1024,0,0}                        { Select mimimum stack for TSR }

Uses DOS, CRT;

Var
   CRC : Byte;

Procedure Do_CRC(X: Byte);
{
   This procedure accumulates the one-wire CRC of bytes passed
   as arguments. The global variable CRC should be initialized
   to zero prior to the first procedure call.
}

Var
   I : Byte;
   F : Boolean;

Begin
   For I := 1 to 8 do Begin
      F := Odd(X xor CRC);
      CRC := CRC shr 1;
      X := X shr 1;
      If F then CRC := CRC xor 140;
   End;
End;

Var
   S : Word;     { Port address of COM port connected to the DS9097 }

Procedure TouchInit;
{
   This procedure sets up the serial port with port address S for
   115200 baud with six data bits, one stop bit, and no parity.
   Hardware interrupts are enabled.
}

Begin
   Port[S + 3] := $81;                               { Set the DLAB }
   Port[S + 0] := $01;                               { Baud rate    }
   Port[S + 1] := $00;                               {    is 115200 }
   Port[S + 3] := $01;                       { 6 dta, 1 stp, no par }
   Port[S + 1] := $01;                            { Data interrupts }
   Port[S + 4] := $0B;                      { RTS, DTR, and OUT2 on }
End;                                                    { TouchInit }

Function TouchReset: Boolean;
{
   This function transmits the one-wire protocol reset sequence.
   This sequence consists of a low pulse lasting a mimimum of 480
   microseconds followed by a high dead time lasting a mimimum of
   480 microseconds. The function returns True if a presence
   pulse occurs during the dead time, otherwise it returns False.
}

Var
   X : Byte;

Begin
   Repeat until Port[S + 5] and $60 = $60;      { Wait for last bit }
   If Odd(Port[S + 5]) then X := Port[S];  { Empty the input buffer }
   Port[S + 3] := $82;                               { Set the DLAB }
   Port[S]     :=  14;                  { Bit rate is 8228.5714 bps }
   Port[S + 3] :=   2;                       { 7 dta, 1 stp, no par }
   Port[S]     := $F8;             { Reset pulse and high dead time }
   Repeat until Port[S + 5] and $60 = $60;      { Wait for last bit }
   If Odd(Port[S + 5]) then TouchReset := (Port[S] <> $78)
   else TouchReset := False;
   Port[S + 3] := $81;                               { Set the DLAB }
   Port[S]     :=   1;                     { Bit rate is 115200 bps }
   Port[S + 3] :=   1;                       { 6 dta, 1 stp, no par }
End;                                                   { TouchReset }

Procedure Send(X: Byte);
{
   This procedure transmits the byte X to the Touch Memory
   connected to the serial port having port address S.
}

Var
   I : Byte;

Begin
   For I := 1 to 8 do Begin
      Repeat until Port[S+5] and $20 = $20;          { Wait for TBE }
      If Odd(X) then Port[S] := $FF else Port[S] := 0;   { Send bit }
      X := X shr 1;                             { Position next bit }
   End;
End;                                                         { Send }

Function Recv: Byte;
{
   This function returns one byte received from the DS9097
   connected to the serial port having port address S. After
   the byte is read, the cumulative CRC is calculated for it.
}

Var
   I, B, X : Byte;

Begin
   Repeat until Port[S+5] and $60 = $60;    { Wait for TBE and TSRE }
   If Odd(Port[S+5]) then B := Port[S];            { Clear receiver }
   For I := 1 to 8 do Begin   { Transfer all eight bits in the byte }
      Port[S] := $FF;                              { Send start bit }
      X := X shr 1;                  { Position byte to receive bit }
      Repeat until Port[S+5] and $60 = $60;   { Wait for TBE & TSRE }
      If Odd(Port[S+5]) then B := Port[S] else B := $FF;  { Get bit }
      If Odd(B) then X := X or $80;         { Put new bit into byte }
   End;
   Do_CRC(X);                            { Accumulate the CRC value }
   Recv := X;                            { Return the received byte }
End;                                                         { Recv }

Var
   T : Byte;

Procedure SerialInt;
Interrupt;

Const
   MT = 10;               { Number of times to try before giving up }

Var
   I, J : Byte;                               { Temporary variables }
   X, Y : Byte;           { Used to save foreground cursor position }
   Good : Boolean;                                   { Success flag }
   Z    : Array[1..6] of Byte;           { Array to save data bytes }

Begin
   Good := False;                           { No data validated yet }
   I := 0;                             { Initialize the try counter }
   Repeat
      If TouchReset then Begin          { Reset signal acknowledged }
         Send($33); CRC := 0;         { Send the DS199x access code }
         If Recv > 0 then Begin              { Family code is valid }
            For J := 1 to 6 do Z[J] := Recv;         { Get the data }
            J := 0;
            If (CRC = $14) and TouchReset then Begin
               Send($F); CRC := 0;         { Send DS914 access code }
               If Recv = 1 then Begin          { Family code is one }
                  For J := 1 to 6 do Z[J] := Recv;   { Get the data }
                  J := 3;                          { Check if early }
                  While (J < 7) and (Z[J] = 0) do Inc(J);   { wafer }
               End;
            End;
            If J = 7 then J := not CRC else J := CRC;   { Check CRC }
            If J = Recv then Begin                 { CRC is correct }
               Good := True;                     { Indicate success }
               Sound(256);                             { Start beep }
               X := WhereX;                    { Record foreground  }
               Y := WhereY;                    {    cursor position }
               For J := 1 to 6 do Begin       { Show six data bytes }
                  GotoXY(75, J);                  { Position cursor }
                  ClrEol;                          { Clear old data }
                  Write(Z[J]);                     { Write new data }
               End;
               GotoXY(X, Y);              { Restore original cursor }
               Delay(250);                  { Beep lasts 1/4 second }
               NoSound;                             { Stop the beep }
            End;
         End;
      End;
      Inc(I);
   Until Good or (I = MT);
   Port[$20] := $20;                         { Re-enable interrupts }
End;                                                    { SerialInt }

Var
   SP : Array[1..4] of Word Absolute $40:0;    { COM port addresses }
   LI : LongInt;                               { Command line value }
   Z  : Word;                                  { Temporary variable }

Begin                                                { Main program }
   LI := 1; S := SP[LI];                          { Default to COM1 }
   If ParamCount > 0 then Begin
      {$R+}
      Val(ParamStr(1), LI, Z);  { Get COM port number from cmd line }
      {$R-}
      If (Z > 0) or (LI < 1) or (LI > 4) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else s := SP[LI];
   End;
   If S = 0 then WriteLn('COM', LI, ' DOES NOT EXIST') else Begin
      T := S shr 8 and 1;            { Set the interrupt identifier }
      TouchInit;                       { Initialize the serial port }
      CheckBreak := False;
      SetIntVec($1B, SaveInt1B);
      SetIntVec(T + $B, @SerialInt);
      Port[$21] := Port[$21] and not (1 shl (T + 3));
      WriteLn('TouchInt Installed');
      Keep(0);
   End;
End.

