{
   In the following pascal code 1-wire I/O is accomplished using the
   serial port of an IBM PC or compatible. The serial port must be capable
   of a 115,200 bps data rate.
}

Const
   SPA : Word = 0;           { Currently active serial port address }

Function TouchReset(N: Byte): Boolean;
{
   This function transmits the one-wire protocol reset sequence to
   the device connected to COM port number N. This sequence consists
   of a low pulse lasting a mimimum of 480 us followed by a high dead
   time lasting a mimimum of 480 us. The function returns True if a
   presence detect pulse occurs during the dead time, otherwise
   it returns False.
}

Const
   Init : Array[1..4] of Boolean = (True, True, True, True);

Var
   S : Array[1..4] of Word Absolute $40:0;
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   F : Boolean;
   X, Y : Byte;

Begin
   SPA := 0; TouchReset := False;                  { Assume failure }
   If (N > 0) and (N < 5) and (S[N] > 0) then Begin  { Legal port # }
      SPA := S[N];                { Save active serial port address }
      If Init[N] then Begin   { Serial port requires initialization }
         Port[SPA + 3] := $83;                       { Set the DLAB }
         Port[SPA]     :=   1;                      { Bit rate is   }
         Port[SPA + 1] :=   0;                      {    115200 bps }
         Port[SPA + 3] :=   3;               { 8 dta, 1 stp, no par }
         Port[SPA + 1] :=   0;                      { No interrupts }
         Port[SPA + 4] :=   3;                     { RTS and DTR on }
         Init[N] := False;               { Initialization completed }
      End;
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do X := Port[SPA];     { Flush input }
      Port[SPA + 3] := $83;                              { Set DLAB }
      port[SPA+1] := 0;                        { Baud rate is 10473 }
      Port[SPA] := 11;
      Port[SPA + 3] := 3;               { 8 data, 1 stop, no parity }
      Port[SPA] := $F0;                      { Send the reset pulse }
      Repeat                 { Wait until character back or timeout }
         Y := Port[SPA + 5];
         F := Odd(Y);
      until F or (T > M);
      If F then X := Port[SPA] else X := $F0;
      If (X <> $F0) Then Begin        { If more bits back than sent }
         TouchReset := True;            {    then there is a device }
         If ((Y and $18) <> 0) Then Begin  { Framing error or break }
            Repeat until Port[SPA + 5] and $60 = $60;  { TBE & TSRE }
            Repeat F := Odd(Port[SPA + 5]) until F or (T > M);
            If F then X := Port[SPA];
         End;
      End;
      Port[SPA + 3] := $83;                          { Set the DLAB }
      Port[SPA]     :=   1;                { Bit rate is 115200 bps }
      Port[SPA + 3] :=   3;                  { 8 dta, 1 stp, no par }
   End;
End;

Function TouchByte(X: Byte): Byte;
{
   This function transmits the byte X to the device attached to the
   currently active serial port SPA, and returns a byte from the
   device as its value.
}

Var
   T    : LongInt Absolute $40:$6C;
   M    : LongInt;
   I, J : Byte;

Begin
   If SPA = 0 then TouchByte := X else Begin
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do I := Port[SPA];     { Flush input }
      I := 0; J := 0;      { Initialize output & input bit counters }
      Repeat
         If Odd(Port[SPA + 5]) then Begin
            Inc(J); If Odd(Port[SPA]) then X := X or $80;
         End else If (I<=J) and (Port[SPA+5] and $20 = $20) then Begin
            If Odd(X) then Port[SPA] := $FF else Port[SPA] := 0;
            X := X shr 1; Inc(I);
         End;
      Until (J = 8) or (T > M);
      While (J < 8) do Begin
         X := X shr 1 or $80;
         Inc(J)
      End;
      TouchByte := X;
   End;
End;

Function TouchBit(X: Boolean): Boolean;
{
   This function transmits the bit corresponding to the boolean
   value of X to the device attached to the currently active serial
   port SPA, and returns the bit received from the device as its
   boolean value.
}

Var
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   F : Boolean;
   I : Byte;

Begin
   If SPA = 0 then TouchBit := X else Begin
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do I := Port[SPA];     { Flush input }
      If X then Port[SPA] := $FF else Port[SPA] := 0;    { Send bit }
      Repeat F := Odd(Port[SPA + 5]) until F or (T > M);  { Receive }
      TouchBit := not F or Odd(Port[SPA]);    { Return received bit }
   End;
End;

{
   The following Pascal code implements the ROM search as shown in the
   flowchart of figure 5-3. (This code does not attempt to check the CRC
   of the ROM data or to detect DS1990s.)
}

Var
   ROM : Array[0..7] of Byte;     { Buffer for collecting ROM data }

Const
   LD : Byte = 0;                               { Last Discrepancy }
   F : Boolean = False;                                { Done Flag }
   PN = 1;                                       { Default to COM1 }

Function Next: Boolean;

Var
   I    : Byte;                                    { ROM Bit Index }
   X    : Byte;                            { (Bit A) + 2 * (Bit B) }
   L0   : Byte;                               { Discrepancy Marker }
   G    : Boolean;                                    { Output Bit }
   J, K : Byte;    { Used to address individual bits in a ROM byte }

Begin
   Next := False;
   If F then F := False else If TouchReset(PN) then Begin
      X := TouchByte($F0); I := 1; J := 0; K := 1; L0 := 0;
      Repeat
         X := Byte(TouchBit(True));
         X := X or Byte(TouchBit(True)) shl 1;
         If X = 3 then J := 8 else Begin
            If X > 0 then G := Odd(X) else Begin
               If I < LD then G := (ROM[J] and K > 0) else G := (I = LD);
               If not G then L0 := I;
            End;
            If G then ROM[J] := ROM[J] or K
            else ROM[J] := ROM[J] and not K;
            G := TouchBit(G);
            Inc(I); K := Byte(K shl 1);
            If K = 0 then Begin
               Inc(J); Inc(K);
            End;
         End;
      Until J = 8;
      If I < 65 then LD := 0 else Begin
         LD := L0; F := (LD = 0); Next := True;
      End;
   End else LD := 0;
End;

Function First: Boolean;

Begin
   LD := 0; F := False;
   First := Next;
End;

Begin
End.
