/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * RD_CLOCK.C - This program uses the Touch Memory routines in TOUCHIO.OBJ
 *            read the rom code of a DOW device on the 1-Wire.
 *
 * Date: May 11, 1992
 * Compiler: Borland C++ 3.1
 * Externals: TOUCHIO.C
 * Version: 1.1
 *
 * Syntax:  RD_CLOCK [(com port #, default 1)]
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned long ulong;

/* Global Variables */
const char TRUE   =  1;
const char FALSE  =  0;
const char X      = -1;
/* Global Variables needed by TOUCHIO.C */
int   com_port;      /* serial com port for DOW */
uint  CRC16;         /* 16-bit cyclic redundancy check */
uchar DOWCRC;        /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* Function Prototypes for 1-wire routines in 'TOUCHIO.C'*/
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar First( void );
extern uchar TouchByte( uchar );
extern uchar Access( void );
extern uchar * RomData( void );
extern uchar dowcrc(uchar);

/* Function Prototype for this module */
void  main(int, char **);
int   Open_Port(int);
int   Read_TimeDate(uchar *);
void  Convert_TimeDate(uchar *, uchar *);
ulong uchar_to_bin(int, uchar *);


/*--------------------------------------------------------------------------
 * This is the main routine in RD_CLOCK.C, it reads the time of a DS1994
 * on the 1-Wire.
 */
void main(int argc, char **argv)
{
     int   flag;
     uchar raw_timedate[33], Prn_Str[50], *ROM;

     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                     RD_CLOCK - Version 1.1\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port number from the command line parameters */
     if (argc != 2) com_port = 1;
     else com_port = atoi(argv[1]);

     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     if (!First())
        printf("                   No Touch Memory on COM%d!\n"
               "    Touch Memory contact required during program execution.\n\n",com_port);
     else
     {
          ROM = RomData();  /* get the pointer to the ROM CODE */

          /* check to make sure part detected is a DS1994 */
          if (ROM[0] != 0x04)
          {
               printf("ERROR: This demo program requires a DS1994.\n");
               exit(1);
          }

          flag = Read_TimeDate(raw_timedate);

          if (flag == TRUE)
          {
               Convert_TimeDate(raw_timedate,Prn_Str);
               printf("The clock on the DS1994 reads: %s\n\n",Prn_Str);
          }
          else if (flag == FALSE-1)
               printf("Contact lost before clock read complete.\n\n");
          else
               printf("A KEY was pressed to abort the clock read.\n\n");
     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 * Read the bytes 1-6 of the 17th page of a DS1994. This contains the
 * control register and the current real-time clock value.
 */
int Read_TimeDate(uchar *tk)
{
     uint i,flag,first=1;
     uchar ch;
     int key=-1;
     tk[0] = 0;

     do
     {
          if (!Access()) /* select the device that was found */
             return FALSE-1;

          TouchByte(0xF0);         /* read memory command */

          TouchByte(0x01);         /* write the target address byte 1 */

          TouchByte(0x02);          /* write the target address byte 2 */

          flag = TRUE;            /* reset flag before loop */

          /* loop to read control register and time/date */
          for (i = 1; i <= 6; i++)
          {
               ch = TouchByte(0xFF);   /* read byte */

               if ( !(!first && (i == 2)) ) /* only read lsbyte once */
               {                            /* because fractional seconds */
                       if (ch != tk[i])     /* may change in multiple reads*/
                          flag = FALSE; /* compare with old set flag*/
                       tk[i] = ch;
               }
          }

          first = FALSE;

          if (kbhit()) key = getch();

     } while ((flag == FALSE) && (key == X));

     if (key != X) return key;

     /* check to see if Oscillator Enable is set */
     if (!(tk[1] & 0x10))
        printf("WARNING: The Oscillator Enable bit in the Control Register is\n"
               "         not set.  The clock will not increment!\n\n");

     if (Access()) return TRUE;
     else return FALSE-1;
}


/*--------------------------------------------------------------------------
 * Convert_TimeDate takes a string 'ptr' of binary seconds and calculates
 * and returns the time/date in standard format.
 */
int dm[] = { 0,0,31,59,90,120,151,181,212,243,273,304,334 };

void Convert_TimeDate(uchar *ptr, uchar *Prn_Str)
{
     uint frsecond,second,minute,hour,day,month,year;
     int tmp,i,j;
     ulong x,y;

     /* get value of number of seconds */
     x = uchar_to_bin(4,&ptr[3]);

     /* get the hundredths of a second */
     frsecond = (99*(ptr[2]+2))/255;

     /* check to make sure date is not over 2020 */
     if (x > 0x5FDD4280)
        x = 0;

     y = x/60;  second = (uint)(x-60*y);
     x = y/60;  minute = (uint)(y-60*x);
     y = x/24;  hour   = (uint)(x-24*y);
     x = 4*(y+731);  year = (uint)(x/1461);
     i = (int)((x-1461*(ulong)(year))/4);

     month = 1 + (i+1)/30;
     tmp = (month > 2) && ((year & 3)==0) ? 1 : 0;
     j = dm[month]+tmp;

     if (i < j)
     {
          month -= 1;
          tmp = (month > 2) && ((year & 3)==0) ? 1 : 0;
          j = dm[month]+tmp;
     }

     day = i-j+1;
     year = (year < 32)  ? year + 68 : year - 32;

     sprintf((char*)(Prn_Str)," %02d/%02d/%02d  %2d:%02d:%02d.%02d ",
             month,day,year,hour,minute,second,frsecond);
}


/*--------------------------------------------------------------------------
 * UCHAR_TO_BIN converts a binary uchar string 'str' into a ulong return
 * number.  'num' indicates the length of the binary uchar string.
 */
ulong uchar_to_bin(int num, uchar *str)
{
     int i;
     ulong l = 0;

     for (i = (num-1); i >= 0; i--)
          l = (l << 8) | (int)(str[i]);

     return l;
}
