/*---------Copyright (C) 1992 Dallas Semiconductor Corporation--------------
 *
 * RD_ENC.C - This program uses the Touch Memory routines in TOUCHIO.OBJ
 *            to read a Ram record of a device on the 1-Wire.  The record
 *            is read from starting page specified on the command line.
 *            The record is assumed to have been encrypted with the utility
 *            program 'WR_ENC'.  The record is decoded accordingly and
 *            displayed.
 *
 * Date: May 11, 1992
 * Compiler: Borland C++ 3.1
 * Externals: TOUCHIO.C
 * Version: 1.1
 *
 * Syntax:  RD_ENC (com port #) (starting page #)
 */

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/* Type definitions */
typedef unsigned char uchar;
typedef unsigned int  uint;

/* Global Variables */
const char TRUE   = 1;
const char FALSE  = 0;
/* Global Variables needed by TOUCHIO.C */
int   com_port;       /* serial com port for DOW */
uint  CRC16;          /* 16-bit cyclic redundancy check */
uchar DOWCRC;         /* 8-bit Dallas Semiconductor cyclic redundancy check*/

/* External Function Prototypes from 'TOUCHIO.C' */
extern uchar Setup( uchar );
extern uchar TouchReset( void );
extern uchar TouchByte( uchar );
extern uchar Access( void );
extern uchar First( void );
extern uchar * RomData( void );
extern uint  crc16(int);

/* Function Prototypes for RD_ENC.C */
void main(int, char **);
int  Open_Port( int );
int  Read_Ram_String(uchar *, uint);
void Decode(uchar *, int);


/*--------------------------------------------------------------------------
 * This is the main routine in RD_RAM.C.
 */
void main(int argc, char **argv)
{
     uchar buf[512], *ROM;
     int len,page,i;

     printf("\n         Dallas Semiconductor PC Touch Memory Utility\n"
            "                     RD_ENC - Version 1.1\n"
            "      Copyright (C) 1992 Dallas Semiconductor Corporation\n\n");

     /* extract the com_port and page number from the command line parameters*/
     if (argc != 3)
     {
           printf("\nSYNTAX: RD_ENC (com port #) (starting page #)\n");
           exit(1);
     }
     else
     {
          com_port = atoi(argv[1]);
          page = atoi(argv[2]);
     }


     /* check to see if the com port is valid */
     if (!Open_Port(com_port))
        exit(1);

     if (!First())
        printf("                   No Touch Memory on COM%d!\n"
               "    Touch Memory contact required during program execution.\n\n",com_port);
     else
     {
          ROM = RomData();  /* get the pointer to the ROM CODE */

          /* check to make sure part detected has the indicated page */
          if (ROM[0] == 0x08 && page >= 4) /* DS1992 has 4 pages */
          {
               printf("\nERROR: The part detected is a DS1992, with a maximum page number of 3!\n");
               exit(1);
          }
          else if (ROM[0] == 0x01 || ROM[0] == 0x02)
          {
               printf("\nERROR: This demo program is written for DS1992,3 and 4's.  The part\n"
                      "         detected is not one of these.\n");
               exit(1);
          }
          else if (page >= 16)
          {
               printf("\nERROR: The maximum page number is 15!\n\n");
               exit(1);
          }

          /* read the ram starting at the selected page */
          len = Read_Ram_String(buf, page << 5);

          if (len > FALSE)
          {
               printf("The following is a decoded record of length %d found starting\n"
                      "on page %d. It is enclosed in {}.\n\n{",len,page);

               Decode(buf,len); /* decode the string */

               for(i = 0; i < len; i++) /* loop to print the chars, * if not valid */
               {
                    if (buf[i] < 0x20 || buf[i] > 0x7E)
                       putchar('*');
                    else
                       putchar(buf[i]);
               }
               printf("}\n\n");
          }
          else if (len == FALSE)
               printf("There is no valid record starting on page %d.\n\n",page);
          else
               printf("Contact was lost before reading was complete.\n\n");
     }
}


/*--------------------------------------------------------------------------
 *  This function opens the selected com port and verifies there is not a
 *  short. Returns TRUE or FALSE.
 */
int Open_Port(int cm_pt)
{
   /*  make sure we can use this port */
   if (!Setup(cm_pt))
   {
      printf("ERROR: The serial port chosen (COM%d), is not valid on this computer!\n",com_port);
      return FALSE;
   }

   /*  either no brick is connected, or com_port is to slow */
   if (TouchReset() == 3)
   {
      printf("ERROR: The serial port brick is not connected to the chosen port (COM%d)!\n",com_port);
      return FALSE;
   }

   return TRUE;
}


/*--------------------------------------------------------------------------
 * This function takes a string (buf) of length (len) and decodes is the
 * manner that it was encrypted in 'WR_ENC.C'.  This method is a simple
 * XOR of the ROM CODE.
 */
void Decode(uchar *buf, int len)
{
   int i, sp=0;
   uchar *ROM;

   ROM = RomData();       /* get a pointer to the ROM CODE */

   for(i = 0; i < len; i++)
   {
      buf[i] ^= 0xAA;         /* XOR with 0xAA */
      buf[i] ^= ROM[sp++];    /* XOR the string with the ROM CODE */
      if (sp == 8) sp = 0;
   }
}


/*--------------------------------------------------------------------------
 * Return up to 508 bytes of the non-volatile ram of a selected 92,93,94
 * starting at a provided address.  Returns the number of the bytes
 * in the record.
 */
int Read_Ram_String(uchar *buf, uint addr)
{
     uint i,len,cnt=0;

     buf[0] = NULL;                 /* zero the output buffer */

     if (!Access())                 /* select the device that was found */
        return FALSE-1;

     TouchByte(0xF0);                /* read memory command */

     TouchByte(addr & 0xFF);         /* write the target address byte 1 */

     TouchByte((addr & 0xFF00) >> 8);/* write the target address byte 2 */

     CRC16 = addr >> 5;              /* seed the CRC16 with page */

     len = TouchByte(0xFF);          /* read the first length byte */

     crc16(len);                     /* compute CRC16 on first byte */

     if (len == 255)
     {
          len = len + TouchByte(0xFF); /* if = 255 then read second length byte */
          crc16(len-255);             /* compute crc16 on second byte */
     }

     if (len > (510-addr))          /* abort read if length to large */
     {
        if (!Access())              /* verify that length was not  */
           return FALSE-1;          /* because connection was lost */
        else
           return FALSE;
     }

     for (i = 0; i < len+2; ++i)    /* loop to read record */
     {
          buf[cnt] = TouchByte(0xFF);
          crc16(buf[cnt++]);
     }

     buf[cnt-2] = NULL;             /* zero end the string over the CRC */

     if (CRC16 == 0xB001)           /* check to see if CRC is correct */
        return cnt-2;               /* return number of byte in record */
     else                           /* minus the length and crc bytes */
     {
        if (!Access())               /* verify that bad string was not */
           return FALSE-1;           /* because connection was lost    */
        else
           return FALSE;
     }
}
