{$C-}            { Allow comments longer than 256 characters }

{$M 0 $1800}                             { Define Memory Map }

Program LCDTime;          { Version 1.01   SMC   May 5, 1992 }
{
   This program reads the date/time from a DS1494, calculates the
   corrected date/time if the calibration constants (alarm
   registers) are nonzero, and displays the results on a liquid
   crystal display.  The time in the DS1494 is assumed to be
   standard time, not daylight savings time.  The program makes
   the correction for daylight savings time as needed, depending
   on the season.  (Results are calculated only to the nearest
   second, allowing use of the long integer data type for all
   calculations.  In order to extend the life of the battery that
   powers the DS1494, the program polls the DS1494 for only about
   1/2 second every minute, using Timer0 to interpolate seconds
   from one minute to the next.  This program works correctly
   only with an 11.0592 MHz crystal.)

   The corrected Date/Time (DT) is calculated as follows:

   Corrected DT  =  Real DT  +  ( Real DT  -  A )  /  B    where

   A = the unsigned integer in the real time alarm register.
   B = the signed integer in the interval time alarm register.

   The real time register and the value of A in the real time
   alarm register can be set to the DOS clock with the PC program
   SetTime, and the value of B can later be computed from the DOS
   time and stored in the interval time alarm register with the
   PC program CalTime.  Either A or B may be recalculated
   independently as needed with the SetTime and CalTime programs.

   The program has been written for compilation with the SYSTEM51
   (Ver 3.10) software development system from KSC Software Systems,
   Ludvig Holsteins Alle 137, DK-2750 Ballerup, Denmark, Phone
   (Int. 45) 44 97 69 11, Fax (Int. 45) 44 97 96 12.
}

Var
   DTA : Boolean at P1.3;     { Define the one-wire data pin }
   GND : Boolean at P1.4;   { Define the one-wire ground pin }

Procedure MapMem;
{
   This procedure sets the memory map of an 8K DS5000 for 6144
   bytes of program memory and 2048 bytes of "external" memory.
}

Begin
   Code
      $75 $C7 $AA                { MOV       TA,       #0AAH }
      $75 $C7 $55                { MOV       TA,       #55H  }
      $75 $87 $00                { MOV       PCON,     #0    }
      $75 $C7 $AA                { MOV       TA,       #0AAH }
      $75 $C7 $55                { MOV       TA,       #55H  }
      $43 $C6 $02                { ORL       MCON,     #2    }
      $75 $C6 $32                { MOV       MCON,     #32H  }
      $75 $C7 $AA                { MOV       TA,       #0AAH }
      $75 $C7 $55                { MOV       TA,       #55H  }
      $53 $C6 $FD;               { ANL       MCON,     #0FDH }
End;

Var
   Control : Byte at $000;            { LCD control register }
   Status  : Byte at $100;             { LCD status register }

Procedure Command(X: Byte);
{
   This procedure sends the command X to the LCD controller.
}

Begin
   Repeat until Status and $80 = 0;
   Control := X;
End;

Procedure ResetLCD;
{
   This procedure initializes the LCD display with no cursor.
}

Begin
   Command($38);                   { Set up the display mode }
   Command($C);             { Turn on display with no cursor }
End;

Procedure ClrScr;
{
   This procedure clears the screen and moves to home.
}

Begin
   Command($1);                           { Clear the screen }
   Command($2);                    { Home to top left corner }
End;

Procedure GotoXY(X, Y: Byte);
{
   This procedure positions the LCD display at (X, Y).
}

Begin
   Case Y of
      2, 3: Y := 5 - Y;
   End;
   Command(32 * Y + X + 95);
End;

Function TouchReset: Boolean;
{
   This function transmits a Reset signal on the one-wire
   bus and returns True if a Presence signal is detected.
}

Begin
   Code
      $74 $04      {           MOV       A,        #4        }
      $C2 DTA      {           CLR       DATA_BIT            }
      $75 $F0 $DD  {           MOV       B,        #221      }
      $D5 $F0 $FD  {           DJNZ      B,        $         }
      $D2 DTA      {           SETB      DATA_BIT            }
      $75 $F0 $06  {           MOV       B,        #6        }
      $C3          {           CLR       C                   }
      $20 DTA $08  { WAITLOW:  JB        DATA_BIT, WH        }
      $D5 $F0 $FA  {           DJNZ      B,        WAITLOW   }
      $D5 $E0 $F7  {           DJNZ      ACC,      WAITLOW   }
      $80 $08      {           SJMP      ABORT               }
      $75 $F0 $6F  { WH:       MOV       B,        #111      }
      $A0 DTA      { HL:       ORL       C,        /DATA_BIT }
      $D5 $F0 $FB  {           DJNZ      B,        HL        }
      $33;         { ABORT:    RLC       A                   }
   TouchReset := ACC.0;
End;

Function TouchByte(X: Byte): Byte;
{
   This function transmits the byte X on the one-wire bus and
   returns the byte received concurrently from the one-wire bus.
}

Begin
   B := X;
   ACC := B;
   Code
      $75 $F0 $08  {           MOV       B,        #8        }
      $C0 $F0      { BIT_LOOP: PUSH      B                   }
      $13          {           RRC       A                   }
      $C2 DTA      {           CLR       DATA_BIT            }
      $75 $F0 $01  {           MOV       B,        #1        }
      $D5 $F0 $FD  {           DJNZ      B,        $         }
      $92 DTA      {           MOV       DATA_BIT, C         }
      $75 $F0 $02  {           MOV       B,        #2        }
      $D5 $F0 $FD  {           DJNZ      B,        $         }
      $A2 DTA      {           MOV       C,        DATA_BIT  }
      $75 $F0 $14  {           MOV       B,        #20       }
      $D5 $F0 $FD  {           DJNZ      B,        $         }
      $D2 DTA      {           SETB      DATA_BIT            }
      $D0 $F0      {           POP       B                   }
      $D5 $F0 $DE  {           DJNZ      B,        BIT_LOOP  }
      $13;         {           RRC       A                   }
   B := ACC;
   TouchByte := B;
End;

Procedure WByte(X: Byte);

Begin
   X := TouchByte(X);
End;

Function RByte: Byte;

Begin
   RByte := TouchByte($FF);
End;

Type
   TRec = Array[0..3] of Byte;

Function TimeIn(Var T: LongInt): Boolean;
{
   This function reads the Date/Time from a DS1494.
   It returns True if the Date/Time is not all ones,
   not all zeros, and agrees with the previous value.
}

Var
  L, M    : LongInt;
  R       : TRec at L;
  F, G, H : Boolean;
  I, X    : Byte;

Begin
  F := TouchReset;
  If F then Begin
     WByte($CC); WByte($F0); WByte(2); WByte(2);
     F := False; G := False; H := (RByte > 127);
     For I := 0 to 3 do Begin
        X := RByte;
        F := (X < $FF) or F;
        G := (X > 0) or G;
        R[I] := X;
     End;
     If H then Inc(L);
     H := (L = M) or (L = M + 1);
     F := F and G and H;
     M := L;
     If F then T := L;
  End;
  TimeIn := F;
End;

Function AlarmIn(Var A, B: LongInt): Boolean;
{
   This function reads the alarm registers from a DS1494.
   It returns True if the alarm registers are not all ones
   and agree with the previous values.
}

Var
  SA, SB, XA, XB : LongInt;
  R              : TRec at XA;
  S              : TRec at XB;
  F, G           : Boolean;
  I, X           : Byte;

Begin
  F := TouchReset;
  If F then Begin
     WByte($CC); WByte($F0); WByte($10); WByte(2);
     F := False; G := (RByte > 127);
     For I := 0 to 3 do Begin
        X := RByte;
        F := (X < $FF) or F;
        R[I] := X;
     End;
     If G then Inc(XA);
     For I := 0 to 3 do Begin
        X := RByte;
        F := (X < $FF) or F;
        S[I] := X;
     End;
     F := (XA = SA) and F;
     F := (XB = SB) and F;
     SA := XA; SB := XB;
     If F then Begin
        A := XA; B := XB;
     End;
  End;
  AlarmIn := F;
End;

Procedure DayToDate(I: LongInt; Var YR: Word; Var MO, DA: Byte);
{
   Convert from Day to Date.
}

Const
   DM : Array[1..13] of Word =
      (0,31,59,90,120,151,181,212,243,273,304,334,365);

Var
   N, J : Word;

Begin
   I := 4 * (I + 731); YR := I div 1461;
   N := (I - 1461 * YR) div 4;
   MO := 1 + (N + 1) div 30;
   J := DM[MO];
   If (MO > 2) and (YR mod 4 = 0) then Inc(J);
   If N < J then Begin
      Dec(MO); J := DM[MO];
      If (MO > 2) and (YR mod 4 = 0) then Inc(J);
   End;
   DA := N - J + 1; YR := YR + 1968;
End;

Procedure Show(X: Byte);
{
   Display data with leading zeros.
}

Begin
   If X < 10 then Write(Display, '0');
   Write(Display, X);
End;

Procedure LCDisplay(T: LongInt);

Const
   Month : Array[1..12] of String[3] = ('JAN', 'FEB', 'MAR', 'APR',
                                        'MAY', 'JUN', 'JUL', 'AUG',
                                        'SEP', 'OCT', 'NOV', 'DEC');
   Week  : Array[0..6] of String[3]  = ('SUN', 'MON', 'TUE', 'WED',
                                        'THU', 'FRI', 'SAT');

Var
   N                  : LongInt;
   HR, MI, SE, MO, DA : Byte;
   YR                 : Word;
   I                  : Integer;
   F                  : Boolean;
   DST                : Boolean;

Begin
   If DST then T := T + 3600;          { Add an hour for DST }
   {
      Calculate Hours, Minutes, and Seconds.
   }
   N := T div 60; SE := T - 60 * N;
   T := N div 60; MI := N - 60 * T;
   N := T div 24; HR := T - 24 * N;
   {
      Calculate Years, Months, and Days.
   }
   DayToDate(N, YR, MO, DA);
   I := (N + 4) mod 7;           { Calculate day of the week }
   {
      Determine whether this is daylight savings time.
   }
   If (MO < 4) or (MO > 10) then DST := False else
   If (MO > 4) and (MO < 10) then DST := True else
   If MO = 4 then Begin
      {
         Check for first Sunday in April.
      }
      If DA <= I then DST := False else
      If I > 0 then DST := True else DST := (HR >= 2);
   End else Begin
      {
         Check for last Sunday in October.
      }
      If DA < I + 25 then DST := True else
      If I > 0 then DST := False else
      If DST then DST := (HR < 2) else DST := (HR < 1);
   End;
   {
      Display the results.
   }
   F := (HR >= 12);
   If F then HR := HR - 12;       { Convert to 12 hour clock }
   If HR = 0 then HR := 12;       {    with AM/PM indicator. }
   GotoXY(1, 1);
   Write(Display, Month[MO], ' ');
   Show(DA); Write(Display, ', ', YR:4, ' ', Week[I]);
   GotoXY(1, 2);
   Show(HR); Write(Display, ':');
   Show(MI); Write(Display, ':');
   Show(SE); Write(Display, ' ');
   If F then Write(Display, 'P') else Write(Display, 'A');
   Write(Display, 'M  ');
   If DST then Write(Display, 'DST') else Write(Display, 'STD');
End;

Procedure StartInt;

Begin
   TL0 := 0; TH0 := $DC;
   TMOD := TMOD and $F0 or 1;
   TR0 := True; ET0 := True; EA := True;
End;

Var
   H : Byte;            { Counter for hundredths of a second }

Procedure Interrupt Timer0;

Begin
   TH0 := $DC;
   Inc(H);
End;

Var
   I          : Byte;
   T, N, A, B : LongInt;

Begin
   MapMem; ResetLCD; ClrScr;
   StartInt; GND := False; T := 0;
   {
      Display the startup message.
   }
   GotoXY(6, 1); Write(Display, 'Dallas');
   GotoXY(3, 2); Write(Display, 'Semiconductor');
   {
      Display the Date/Time indefinitely.
   }
   While True do Begin
      Repeat
         N := T; H := 0;
         Repeat
            If H > 99 then Begin
               ClrScr; Write(Display, 'Please connect a');
               GotoXY(2, 2); Write(Display, 'TIME-IN-A-CAN!');
               Repeat until TouchReset; H := 0;
            End;
         Until TimeIn(T) and AlarmIn(A, B);
         {
            Calculate correction to the Date/Time.
         }
         If (A > 0) and (B <> 0) then T := T + (T - A) div B;
      Until T > N;
      H := 100;
      For I := 1 to 60 do Begin
         Repeat until H > 99; H := 0;
         {
            Display the results.
         }
         LCDisplay(T);
         Inc(T);
      End;
      Repeat until H > 49;
      Dec(T);
   End;
End.
