Program NewLock;                      { Version 1.00  SMC  03/12/92 }
{
   This program demonstrates the use of Touch Memory and the DS9097
   Serial Port Adapter to effectively lock the keyboard of a PC.
   The program is activated as a TSR (Terminate and Stay Resident)
   program.  To activate the program with the DS9097 attached to
   COM1 or COM2, type

   NEWLOCK

   or

   NEWLOCK 2

   To lock the PC, touch any Touch Memory with the probe.  This will
   lock the PC.  To unlock the PC, touch the same Touch Memory again.
   (The program "imprints" on the first Touch Memory it detects, so
   that only that memory can be used for locking and unlocking.)

   While the PC is locked, the screen boarder changes colors
   continuously as an indicator.

   This program uses the serial interrupt generated by the Touch
   Memory presence pulse to detect the Touch Memory.  Therefore,
   the program is activated only when a Touch Memory makes contact
   with the probe.  The program does not waste CPU time polling
   when no Touch Memory is present.  Because it uses interrupts,
   the program may be used only with COM1 or COM2.
}

{$M 1024,0,0}                                { Select minimum stack }

Uses DOS, CRT;                           { Required Pascal I/O unit }

Var
   SPA : Word;               { Currently active serial port address }

Function TouchReset(N: Byte): Boolean;
{
   This function transmits the one-wire protocol reset sequence to
   the device connected to COM port number N.  This sequence consists
   of a low pulse lasting a mimimum of 480 us followed by a high dead
   time lasting a mimimum of 480 us.  The function returns True if a
   presence detect pulse occurs during the dead time, otherwise
   it returns False.
}

Const
   Init : Array[1..4] of Boolean = (True, True, True, True);

Var
   S : Array[1..4] of Word Absolute $40:0;
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   F : Boolean;
   X : Byte;

Begin
   SPA := 0; TouchReset := False;                  { Assume failure }
   If (N > 0) and (N < 5) and (S[N] > 0) then Begin  { Legal port # }
      SPA := S[N];                { Save active serial port address }
      If Init[N] then Begin   { Serial port requires initialization }
         Port[SPA + 3] := $83;                       { Set the DLAB }
         Port[SPA]     :=   1;                      { Bit rate is   }
         Port[SPA + 1] :=   0;                      {    115200 bps }
         Port[SPA + 3] :=   3;               { 8 dta, 1 stp, no par }
         Port[SPA + 1] :=   1;                 { Interrupts allowed }
         Port[SPA + 4] :=  $B;               { RTS, DTR and OUT2 on }
         Init[N] := False;               { Initialization completed }
      End;
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do X := Port[SPA];     { Flush input }
      Port[SPA + 3] := $82;                          { Set the DLAB }
      Port[SPA]     :=  18;                  { Bit rate is 6400 bps }
      Port[SPA + 3] :=   2;                  { 7 dta, 1 stp, no par }
      Port[SPA]     := $F8;                     { Send reset signal }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      Repeat F := Odd(Port[SPA + 5]) until F or (T > M);
      TouchReset := F and (Port[SPA] <> $78);     { Return presence }
      Port[SPA + 3] := $83;                          { Set the DLAB }
      Port[SPA]     :=   1;                { Bit rate is 115200 bps }
      Port[SPA + 3] :=   3;                  { 8 dta, 1 stp, no par }
   End;
End;

Function TouchByte(X: Byte): Byte;
{
   This function transmits the byte X to the device attached to the
   currently active serial port SPA, and returns a byte from the
   device as its value.
}

Var
   T    : LongInt Absolute $40:$6C;
   M    : LongInt;
   I, J : Byte;

Begin
   If SPA = 0 then TouchByte := X else Begin
      M := T + 1;                       { Initialize the time limit }
      Repeat until Port[SPA + 5] and $60 = $60;  { Await TBE & TSRE }
      While Odd(Port[SPA + 5]) do X := Port[SPA];     { Flush input }
      I := 0; J := 0;      { Initialize output & input bit counters }
      Repeat
         If Odd(Port[SPA + 5]) then Begin
            Inc(J); If Odd(Port[SPA]) then X := X or $80;
         End else If (I<=J) and (Port[SPA+5] and $20 = $20) then Begin
            If Odd(X) then Port[SPA] := $FF else Port[SPA] := 0;
            X := X shr 1; Inc(I);
         End;
      Until (J = 8) or (T > M);
      While (J < 8) do Begin
         X := X shr 1 or $80;
         Inc(J)
      End;
      TouchByte := X;
   End;
End;

Var
   CRC : Byte;

Procedure Do_CRC(X: Byte);
{
   This procedure accumulates the one-wire CRC of bytes passed
   as arguments.  The global variable CRC should be initialized
   to zero prior to the first procedure call.
}

Var
   I : Byte;
   F : Boolean;

Begin
   For I := 1 to 8 do Begin
      F := Odd(X xor CRC);
      CRC := CRC shr 1;
      X := X shr 1;
      If F then CRC := CRC xor 140;
   End;
End;

Function RByte: Byte;
{
   This function returns the byte received from a Touch Memory.
}

Var
   X : Byte;

Begin
   X := TouchByte($FF);
   Do_CRC(X); RByte := X;
End;

Procedure WByte(X: Byte);
{
   This procedure transmits the byte X to a Touch Memory.
}

Begin
   X := TouchByte(X);
End;

Type
   RomTyp = Array[0..7] of Byte;

Var
   K   : Byte;
   ROM : RomTyp;

Function ReadRom : Boolean;
{
   This function attempts to read the ROM data of a
   Touch Memory for N/18 of a second.  It returns True
   if ROM data with a valid CRC is detected during that
   time interval.
}

Const
   N = 3;

Var
   T : LongInt Absolute $40:$6C;
   M : LongInt;
   I : Byte;

Begin
   M := T + N;
   Repeat
      CRC := $FF;
      If TouchReset(K) then Begin
         WByte($33); CRC := 0;
         For I := 0 to 7 do ROM[I] := RByte;
         If (CRC = $C9) and TouchReset(K) then Begin
            WByte($F); CRC := 0;
            For I := 0 to 7 do ROM[I] := RByte;
         End;
      End;
   Until (CRC = 0) or (T > M);
   ReadRom := (CRC = 0);
End;

Procedure GoAway;
{
   This procedure returns when a Touch Memory has been
   removed for a continuous period of N/18 of a second.
}

Const
   N = 3;

Var
   T : LongInt Absolute $40:$6C;
   M : LongInt;

Begin
   M := T;
   Repeat
      If TouchReset(K) then M := T;
   Until T > M + N;
End;

Procedure Color(X: Byte);
{
   This procedure displays the color indicated by
   X as the boarder color of the display screen.
}

Var
   R : Registers;

Begin
   With R do Begin
      AX := $1001;
      BH := X and $1F;
   End;
   Intr($10, R);
End;

Procedure SerialInt; Interrupt;
{
   This procedure is activated when a presence pulse
   from a Touch Memory generates a serial interrupt.
}

Const
   N = 3;
   Match : RomTyp = (0, 0, 0, 0, 0, 0, 0, 0);
   Mated : Boolean = False;

Var
   T    : LongInt Absolute $40:$6C;
   M    : LongInt;
   I, J : Byte;
   C    : Char;

Begin
   Inline($FB);
   If ReadRom then Begin
      If not Mated then Begin
         Match := ROM;
         Mated := True;
      End;
      I := 0;
      While (I < 8) and (ROM[I] = Match[I]) do Inc(I);
      If I < 8 then Begin
         Sound(100); Delay(100); NoSound;
      End;
      If I = 8 then Begin
         Sound(440); Delay(100); NoSound; GoAway;
         M := T; J := 1;
         Repeat
            Repeat
               If T > M + N then Begin
                  Color(J); Inc(J);
                  If J > 31 then J := 1;
                  M := T;
               End;
               While KeyPressed do C := ReadKey;
            Until ReadRom;
            I := 0;
            While (I < 8) and (ROM[I] = Match[I]) do Inc(I);
            If I < 8 then Begin
               Sound(100); Delay(100); NoSound;
            End;
         Until I = 8;
         Sound(256); Delay(100); NoSound;
         Color(0); GoAway;
      End;
   End;
   Port[$20] := $20;
End;

Var
   N  : Word;
   L  : LongInt;
   SP : Array[1..4] of Word Absolute $40:0;

Begin
   If ParamCount > 0 then Begin
      {$R+} Val(ParamStr(1), L, N); {$R-}
      If (N > 0) or (L < 1) or (L > 2) then Begin
         WriteLn('INCORRECT COMMAND LINE PARAMETER');
         Halt;
      End else K := L
   End else K := 1;
   SPA := SP[K];
   If SPA = 0 then WriteLn('COM', K, ' DOES NOT EXIST') else Begin
      If TouchReset(K) then ;
      N := SPA shr 8 and 1;
      CheckBreak := False;
      SetIntVec($1B, SaveInt1B);
      SetIntVec(N + $B, @SerialInt);
      Port[$21] := Port[$21] and not (1 shl (N + 3));
      WriteLn('NewLock Installed');
      Keep(0);
   End;
End.
